/*      SKQSplines und SKISplines                               */
/*      Burkhard Lenze                                          */
/*      30.06.2000                                              */

#include <stdio.h>
#include <math.h>

typedef long double feld[11];
void dateny(void);
long double f(long double);
void datenf(void);
long double bspline(long double);
void glsloes (void);
long double skq(long double,int,feld);
long double ski(long double,int,feld);
feld y,delta;
int n;

void dateny(void)
{
/* Verzicht auf Einleseroutine und direkter Zugriff auf Aufgabendaten des Skripts */
        n=4;
        y[0]=1.0;
        y[1]=0.25;
        y[2]=0.0;
        y[3]=0.25;
        y[4]=1.0;
}

long double f(long double x)
{
        return(4.0*(x-0.5)*(x-0.5));
}

void datenf(void)
{
/* Verzicht auf Einleseroutine und direkter Zugriff auf Aufgabendaten des Skripts */
        int k;
        long double d;
        n=4;
        d=1.0/n;
        for(k=0;k<=n;k++) y[k]=f(k*d);
}

long double bspline(long double x)
{
        long double bspline=0.0;
        if (x>-2.0) bspline=(((1.0/6.0)*x+1.0)*x+2.0)*x+(4.0/3.0);
        if (x>-1.0) bspline=(-0.5*x-1.0)*x*x+(2.0/3.0);
        if (x> 0.0) bspline=(0.5*x-1.0)*x*x+(2.0/3.0);
        if (x> 1.0) bspline=(((-1.0/6.0)*x+1.0)*x-2.0)*x+(4.0/3.0);
        if (x> 2.0) bspline=0.0;
        return (bspline);
}

void glsloes (void)
{
        int k;
        feld hd,yy;
        /* Initialisierung */
        for (k=0;k<=n;k++)
        {
                yy[k]=y[k];
                hd[k]=2.0/3.0;
        }
        /* Herausschlagen der unteren Nebendiagonalen */
        for (k=1;k<=n;k++)
        {
                hd[k]=hd[k]-(1.0/(36.0*hd[k-1]));
                yy[k]=yy[k]-yy[k-1]/6.0/hd[k-1];
        }
        /* Aufrollen des Systems von unten */
        delta[n]=yy[n]/hd[n];
        for (k=1;k<=n;k++) delta[n-k]=(yy[n-k]-(delta[n-k+1]/6.0))/hd[n-k];
}

long double skq(long double x, int n, feld y)
{
        int k;
        long double help=0.0;
        for(k=0;k<=n;k++) help=help+y[k]*bspline(n*x-k);
        return(help);
}

long double ski (long double x, int n, feld delta)
{
        int k;
        long double help=0.0;
        for(k=0;k<=n;k++) help=help+delta[k]*bspline(n*x-k);
        return(help);
}

/* Hauptprogramm */
int main(void)
{
        int k;
        long double x,z;
        /* Ausgabe der jeweiligen de Boor-Koeffizienten */
        dateny(); /* datenf() oder dateny() */
        glsloes();
        printf("\n");
        for (k=0; k<=n; k++)
                printf("Der %i-te de Boor-Koeffizient von SKQ lautet %.12LE.\n",k,y[k]);
        printf("\n");
        for (k=0; k<=n; k++)
                printf("Der %i-te de Boor-Koeffizient von SKI lautet %.12LE.\n",k,delta[k]);
        printf("\n");
        /* Einige Testausgaben */
        for (k=0; k<=n; k++)
        {
                x=(long double)k/n;
                z=skq(x,n,y);
                printf("SKQ(%.12LE) = %.12LE\n", x, z);
        }
        printf("\n");
        for (k=0; k<=n; k++)
        {
                x=(long double)k/n;
                z=ski(x,n,delta);
                printf("SKI(%.12LE) = %.12LE\n", x, z);
        }
        return(0);
}
