/*      SKQvolK und SKIvolK fr Tragflgeldaten                 */
/*      Burkhard Lenze                                          */
/*      30.06.2000                                              */

#include <stdio.h>
#include <math.h>

typedef long double feld[11];
void datenx(void);
void dateny(void);
long double bspline(long double);
void glsloesx(void);
void glsloesy(void);
long double SKQvolKx(long double,int,feld);
long double SKQvolKy(long double,int,feld);
long double SKIvolKx(long double,int,feld);
long double SKIvolKy(long double,int,feld);
feld x,y,deltax,deltay;
int n;

void datenx(void)
{
/* Verzicht auf Einleseroutine und direkter Zugriff auf Aufgabendaten des Skripts */
        n=7;
        x[0]=0.0;
        x[1]=2.0;
        x[2]=4.0;
        x[3]=7.0;
        x[4]=8.0;
        x[5]=6.0;
        x[6]=4.0;
        x[7]=0.0;
}

void dateny(void)
{
/* Verzicht auf Einleseroutine und direkter Zugriff auf Aufgabendaten des Skripts */
        n=7;
        y[0]=2.0;
        y[1]=3.75;
        y[2]=5.25;
        y[3]=6.5;
        y[4]=5.5;
        y[5]=4.75;
        y[6]=3.75;
        y[7]=1.5;
}

long double bspline(long double t)
{
        long double bspline=0.0;
        if (t>-2.0) bspline=(((1.0/6.0)*t+1.0)*t+2.0)*t+(4.0/3.0);
        if (t>-1.0) bspline=(-0.5*t-1.0)*t*t+(2.0/3.0);
        if (t> 0.0) bspline=(0.5*t-1.0)*t*t+(2.0/3.0);
        if (t> 1.0) bspline=(((-1.0/6.0)*t+1.0)*t-2.0)*t+(4.0/3.0);
        if (t> 2.0) bspline=0.0;
        return (bspline);
}

void glsloesx(void)
{
        int k;
        feld hd,xx;
        /* Initialisierung */
        for (k=0;k<=n;k++)
        {
                xx[k]=x[k];
                hd[k]=2.0/3.0;
        }
        xx[0]=x[0]+(1.0/3.0)*(x[1]-x[0]);
        xx[n]=x[n]-(1.0/3.0)*(x[n]-x[n-1]);
        /* Herausschlagen der unteren Nebendiagonalen */
        hd[1]=hd[1]-(1.0/(18.0*hd[0]));
        xx[1]=xx[1]-(xx[0]/(6.0*hd[0]));
        for (k=2;k<=n-1;k++)
        {
                hd[k]=hd[k]-(1.0/(36.0*hd[k-1]));
                xx[k]=xx[k]-xx[k-1]/(6.0*hd[k-1]);
        }
        hd[n]=hd[n]-(1.0/(18.0*hd[n-1]));
        xx[n]=xx[n]-(xx[n-1]/(3.0*hd[n-1]));
        /* Aufrollen des Systems von unten */
        deltax[n]=xx[n]/hd[n];
        for (k=1;k<=n-1;k++) 
                deltax[n-k]=(xx[n-k]-(deltax[n-k+1]/6.0))/hd[n-k];
        deltax[0]=(xx[0]-(deltax[1]/3.0))/hd[0];
}

void glsloesy(void)
{
        int k;
        feld hd,yy;
        /* Initialisierung */
        for (k=0;k<=n;k++)
        {
                yy[k]=y[k];
                hd[k]=2.0/3.0;
        }
        yy[0]=y[0]+(1.0/3.0)*(y[1]-y[0]);
        yy[n]=y[n]-(1.0/3.0)*(y[n]-y[n-1]);
        /* Herausschlagen der unteren Nebendiagonalen */
        hd[1]=hd[1]-(1.0/(18.0*hd[0]));
        yy[1]=yy[1]-(yy[0]/(6.0*hd[0]));
        for (k=2;k<=n-1;k++)
        {
                hd[k]=hd[k]-(1.0/(36.0*hd[k-1]));
                yy[k]=yy[k]-yy[k-1]/(6.0*hd[k-1]);
        }
        hd[n]=hd[n]-(1.0/(18.0*hd[n-1]));
        yy[n]=yy[n]-(yy[n-1]/(3.0*hd[n-1]));
        /* Aufrollen des Systems von unten */
        deltay[n]=yy[n]/hd[n];
        for (k=1;k<=n-1;k++) 
                deltay[n-k]=(yy[n-k]-(deltay[n-k+1]/6.0))/hd[n-k];
        deltay[0]=(yy[0]-(deltay[1]/3.0))/hd[0];
}

long double SKQvolKx(long double t, int n, feld x)
{
        int k;
        long double helpx=0.0;
        for(k=0;k<=n;k++) helpx=helpx+x[k]*bspline(n*t-k);
        helpx=helpx+(2.0*x[0]-x[1])*bspline(n*t-(-1.0));
        helpx=helpx+(2.0*x[n]-x[n-1])*bspline(n*t-(n+1.0));
        return(helpx);
}

long double SKQvolKy(long double t, int n, feld y)
{
        int k;
        long double helpy=0.0;
        for(k=0;k<=n;k++) helpy=helpy+y[k]*bspline(n*t-k);
        helpy=helpy+(2.0*y[0]-y[1])*bspline(n*t-(-1.0));
        helpy=helpy+(2.0*y[n]-y[n-1])*bspline(n*t-(n+1.0));
        return(helpy);
}

long double SKIvolKx(long double t, int n, feld deltax)
{
        int k;
        long double helpx=0.0;
        for(k=0;k<=n;k++) helpx=helpx+deltax[k]*bspline(n*t-k);
        helpx=helpx+(deltax[1]-2.0*(x[1]-x[0]))*bspline(n*t-(-1.0));
        helpx=helpx+(deltax[n-1]+2.0*(x[n]-x[n-1]))*bspline(n*t-(n+1.0));
        return(helpx);
}

long double SKIvolKy(long double t, int n, feld deltay)
{
        int k;
        long double helpy=0.0;
        for(k=0;k<=n;k++) helpy=helpy+deltay[k]*bspline(n*t-k);
        helpy=helpy+(deltay[1]-2.0*(y[1]-y[0]))*bspline(n*t-(-1.0));
        helpy=helpy+(deltay[n-1]+2.0*(y[n]-y[n-1]))*bspline(n*t-(n+1.0));
        return(helpy);
}

/* Hauptprogramm */
int main(void)
{
        int k,nn=100;
        long double t,xx,yy;
        FILE *dat1, *dat2;
        char datenskq[20], datenski[20];
        /* Ausgabe der jeweiligen de Boor-Koeffizienten */
        datenx();
        dateny();
        glsloesx();
        glsloesy();
        for (k=0; k<=n; k++) 
            printf("Der %i-te de Boor-Koeffizient von SKQvolK lautet %.12LE , %.12LE. \n",k,x[k],y[k]);
        printf("\n");
        for (k=0; k<=n; k++) 
            printf("Der %i-te de Boor-Koeffizient von SKIvolK lautet %.12LE , %.12LE. \n",k,deltax[k],deltay[k]);
        printf("\n");
        /* Einige Testausgaben */
        for (k=0; k<=n; k++)
        {
                t=(long double)k/n;
                xx=SKQvolKx(t,n,x);
                yy=SKQvolKy(t,n,y);
                printf("SKQvolK(%.12LE) = ( %.12LE ,  %.12LE ) \n", t,xx,yy);
        }
        printf("\n");
        for (k=0; k<=n; k++)
        {
                t=(long double)k/n;
                xx=SKIvolKx(t,n,deltax);
                yy=SKIvolKy(t,n,deltay);
                printf("SKIvolK(%.12LE) = ( %.12LE , %.12LE ) \n", t,xx,yy);
        }
        /* Einige weiter umfangreichere Testausgaben in Dateien zur bergabe an Plot-Programm */
        dat1=fopen("datenskq","w");
        dat2=fopen("datenski","w");
        for (k=0; k<=nn; k++)
        {
                t=(long double)k/nn;
                xx=SKQvolKx(t,n,x);
                yy=SKQvolKy(t,n,y);
                fprintf(dat1," %.3Lf   %.3Lf \n", xx,yy);
        }
        for (k=0; k<=nn; k++)
        {
                t=(long double)k/nn;
                xx=SKIvolKx(t,n,deltax);
                yy=SKIvolKy(t,n,deltay);
                fprintf(dat2," %.3Lf   %.3Lf  \n", xx,yy);
        }
        fclose(dat2);
        fclose(dat1);
        return(0);
}
